/*! \file a2d.c \brief Analog-to-Digital converter function library. */
//*****************************************************************************
//
// File Name	: 'a2d.c'
// Title		: Analog-to-digital converter functions
// Author		: Pascal Stang - Copyright (C) 2002
// Created		: 2002-04-08
// Revised		: 2002-09-30
// Version		: 1.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

#include <avr/io.h>
//#include <avr/signal.h>
#include <avr/interrupt.h>

#include "global.h"
#include "a2d.h"

// global variables

//! Software flag used to indicate when
/// the a2d conversion is complete.
volatile uint8_t a2dCompleteFlag;

// functions

// initialize a2d converter
void a2dInit(void)
{

	ADCSRA =  (1<<ADFR);			// free running convert mode
	a2dSetPrescaler(ADC_PRESCALE);	// set default prescaler
	a2dSetReference(ADC_REFERENCE);	// set default reference

	//ADMUX &= ~(1<< ADLAR);		// set to right-adjusted result
	ADCSRA |= (1<<ADIE);			// enable ADC interrupts
	ADCSRA |= (1<<ADEN); 			// enable ADC (turn on ADC power)
	


	a2dCompleteFlag = FALSE;		// clear conversion complete flag
	sei();							// turn on interrupts (if not already on)
}





// turn off a2d converter
void a2dOff(void)
{
	ADCSRA &= ~(1<< ADIE);			// disable ADC interrupts
	ADCSRA &= ~(1<< ADEN);			// disable ADC (turn off ADC power)

}

// configure A2D converter clock division (prescaling)
void a2dSetPrescaler(uint8_t prescale)
{
	
	ADCSRA = (ADCSRA & ~ADC_PRESCALE_MASK) | prescale ;
	//outb(ADCSR, ((inb(ADCSR) & ~ADC_PRESCALE_MASK) | prescale));
}

// configure A2D converter voltage reference
void a2dSetReference(uint8_t ref)
{
	ADMUX = (ADMUX & ~ADC_REFERENCE_MASK) | (ref<<6) ;
	//outb(ADMUX, ((inb(ADMUX) & ~ADC_REFERENCE_MASK) | (ref<<6)));
}

// sets the a2d input channel
void a2dSetChannel(uint8_t ch)
{
	ADMUX = (ADMUX & ~ADC_MUX_MASK) | (ch & ADC_MUX_MASK) ;	// set channel
	//outb(ADMUX, (inb(ADMUX) & ~ADC_MUX_MASK) | (ch & ADC_MUX_MASK));	// set channel
}

// start a conversion on the current a2d input channel
void a2dStartConvert_freerun(uint8_t ch)
{
	ADMUX = ch;						// select channel
	ADC_DDR  &= ~ (1<<ch);		    // PIN als Eingang
    ADC_PORT &= ~ (1<<ch);         // Pullup-Widerstand deaktivieren            	
	//ADCSRA |= (1<<ADIF);			// 
	ADCSRA |= (1<<ADSC);			// start conversion
	
}

/*
// return TRUE if conversion is complete
uint8_t a2dIsComplete(void)
{
	return bit_is_set(ADCSR, ADSC);
}
*/


// starts conversion, waits until conversion is done, and returns result
uint16_t a2dConvert_freerun(void)
{
	a2dCompleteFlag = FALSE;				// clear conversion complete flag
	
	return ADC;	// read ADC (full 10 bits);
}


// 
uint8_t a2dconvert_status(void)
{
	return a2dCompleteFlag;	// read ADC (full 10 bits);
}




/*
// Perform a 8-bit conversion.
// starts conversion, waits until conversion is done, and returns result
uint8_t a2dConvert8bit(uint8_t ch)
{
	// do 10-bit conversion and return highest 8 bits
	return a2dConvert_freerun(ch)>>2;			// return ADC MSB byte
}
*/

/*
//! Interrupt handler for ADC complete interrupt.
SIGNAL(SIG_ADC)
{
	// set the a2d conversion flag to indicate "complete"
	a2dCompleteFlag = TRUE;
}
*/
