//############################################################################
//
// File Name	: 'ad8555.h'
// Title		: AD8555 - Digitally Programmable Sensor Signal Zero-Drift 
//                         Amplifier
// Author		: Zoltan Gradwohl
// Created		: 09/08/2006
// Revised		: 09/08/2006
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
//	Funktionen zur Ansteuerung des AD8555 digital programmierbagen 
//  Brueckensensor Instrumentenverstaerkers
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//############################################################################

#ifndef AD8555_H
#define AD8555_H

#include "global.h"


// alte schaltung :
// #define PIN_AD8555     			PB1		// SDA Pin   
// #define PORT_AD8555        		PORTB 	// SDA Port


#define PIN_AD8555     			6		// SDA Pin   
#define PORT_AD8555        		PORTD 	// SDA Port

#define AD8555_TW0        		3 	// Pulse Width for Loading 0 into Shift Register (Between 0.050 us and 10 us)
#define AD8555_TW1        		155 	// Pulse Width for Loading 1 into Shift Register (>50 us )
#define AD8555_TWS        		20 	// Width between Pulses (> 10 us)


// 38-bit serial word 
#define NSERIAL_WORD_BITS 			38

// Field 0   Bits 0 to 11    12-Bit Start of Packet 1000 0000 0001
#define AD8555_START_OF_PACKET	 	0x0801UL
// Field 1   Bits 12 to 13   2-Bit Function
#define F_CHANGE_SENDSE_CURRENT	 	0x00  // Change Sense Current
#define F_SIMULATE_PARAM_VALUE	 	0x01  // Simulate Parameter Value
#define F_PROGRAM_PARAM_VALUE	 	0x02  // Program Parameter Value
#define F_READ_PARAM_VALUE	     	0x03  // Read Parameter Value
// Field 2   Bits 14 to 15   2-Bit Parameter
#define P_SECOND_STAGE_GAIN_CODE 	0x00  // Second Stage Gain Code
#define P_FIRST_STAGE_GAIN_CODE	 	0x01  // First Stage Gain Code
#define P_OUTPUT_OFFSET_CODE	 	0x02  // Output Offset Code
#define P_OTHER_FUNCTIONS	     	0x03  // Other Functions
// Field 3    Bits 16 to 17  2-Bit Dummy 10
#define AD8555_DUMMY	         	0x02  // Other Functions
// Field 4    Bits 18 to 25  8-Bit Value
#define MASK_SECOND_STAGE_GAIN_CODE 0x07  // 3 LSBs Used
#define MASK_FIRST_STAGE_GAIN_CODE	0x7F  // 7 LSBs Used
#define MASK_OUTPUT_OFFSET_CODE	 	0xFF  // All 8 Bits Used
#define MASK_OTHER_FUNCTIONS	    0x07  // Other Functions
	#define B0_MASTER_FUSE	    	0x01  // Master Fuse
	#define B1_PRODUCTION_TEST_FUSE 0x02  // Fuse for Production Test at Analog Devices
	#define B2_PARITY_FUSE			0x03  // Parity Fuse
// Field 5    Bits 26 to 37  12-Bit End of Packet 0111 1111 1110
#define AD8555_END_OF_PACKET	 	0x07FEUL



// Prototypes
void ad8555_Init(void);
void ad8555_write(uint8_t ad_fun, uint8_t ad_par, uint8_t ad_val);
void send_onewirebus(uint8_t nbits, uint64_t data);
uint64_t mirror64bits(uint8_t nbits, uint64_t in) ;



#endif
