//############################################################################
//
// File Name	: 'hal815.c'
// Title		: Hal815 - programmierbarer linearer Magnetfeldsensor 
//                         
// Author		: Zoltan Gradwohl
// Created		: 03/10/2008
// Revised		: 03/10/2008
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
//
//
//
//############################################################################

#include <avr/io.h>
#include <avr/interrupt.h>
#include "hal815.h"
#include "timer.h"
#include "global.h"			// include our global settings
#include "vt100.h"			// include VT100 terminal commands
#include "rprintf.h"		// include printf function library


uint32_t get_ack_lenght_in_cpu_cycles=0;
volatile uint32_t get_sig_lenght_in_cpu_cycles=0;
volatile uint8_t timer1_measurement_running = 0;
volatile uint8_t in_bit_counter=0;
volatile uint16_t ack_lenght_in_ms;
volatile uint16_t hal815_data_out_=0;

//############################################################################
//! enable and initialize 
void hal815_init(void)
//############################################################################
{
	// initialize input pin
	DDR(PORT_HAL815_IN)		    &= ~(1<<PIN_HAL815_IN);			// set pin as input
	OUT(PORT_HAL815_IN)		    &= ~(1<<PIN_HAL815_IN);			// disable pull-ups
	// initialize output pin1 (5 or 8V)
	DDR(PORT_HAL815_OUT_WRITE)  |=  (1<<PIN_HAL815_OUT_WRITE);		// set pin as output
	OUT(PORT_HAL815_OUT_WRITE)  &= ~(1<<PIN_HAL815_OUT_WRITE);	    // set pin to 0
	// initialize output pin2 (12V)
	DDR(PORT_HAL815_OUT_EEPROM) |=  (1<<PIN_HAL815_OUT_EEPROM);	// set pin as output
	OUT(PORT_HAL815_OUT_EEPROM) &= ~(1<<PIN_HAL815_OUT_EEPROM);	// set pin to 0

}





//############################################################################
//! write data to device 
uint8_t hal815_com(uint8_t hal815_com, uint8_t hal815_adr, uint16_t hal815_data_in, uint16_t* hal815_data_out)
//############################################################################
{
	uint32_t ack_lenght_in_cpu_cycles=0;
//  uint8_t first_reading=0, second_reading=0; 
//	uint8_t i=0;
//	uint32_t read_test_signal_1=0, read_test_signal_2=0, read_test_signal_3=0;
	uint8_t errflag = 0;
	uint32_t serialword 			 = (uint32_t)(0x000000);  
	
	// connect the commands to one qbyte 
	serialword |= (uint32_t)(          	hal815_com  & 0x07)					<<(32-3);  			// 3bits	Command Bits (COM)
	serialword |= (uint32_t)( (UNEVEN(	hal815_com  & 0x07,3))   	& 0x01)	<<(32-3-1);			// 1bit		Command Parity Bit (CP)
	serialword |= (uint32_t)(          	hal815_adr  & 0x0F)			  		<<(32-3-1-4);			// 4bits	Address Bits (ADR)
	serialword |= (uint32_t)( (UNEVEN(	hal815_adr  & 0x0F,4))   	& 0x01)	<<(32-3-1-4-1);		// 1bits	Address Parity Bit (AP)
	
	
	if((hal815_com & 0x07) == COMMAND_READ)
	{
		send_voltagemodulation(NSERIAL_WORD_BITS_READ, serialword);
		
		#define MEASURE_TIME_RECEIVE_SIGNAL
		#ifndef MEASURE_TIME_RECEIVE_SIGNAL
			
		if(hal815_wait4ack(&ack_lenght_in_cpu_cycles))
		{
			errflag = 1;	// write failed
		}
		else
		{
			*hal815_data_out = 0;
			
			// OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE 
			// ----------------------------------------------------------------------------
			// A logical 0 is coded as no voltage change within the
			// bit time. A logical 1 is coded as a voltage change
			// between 50% and 80% of the bit time. After each bit, a
			// voltage change occurs.
			// 
			// - For the first reading the state of the input pin is 
			//   checked after 25% of the complete pulse time (tp0)
			// - After further 60% of the pulse time the input pin is 
			//   checked again
			//   (if both signals are unequal, we have a 1, else a 0)
			// - For the following 13 signals we need instead of the 
			//   25% a third delay time: This is calulated by the 
			//   formula (1.00-0.85+0.25)
			// The values for checking the signal (0.25% and 0.85%) are 
			// estimated 
			// ----------------------------------------------------------------------------
			//                                                                     
			//                  tp0 (ACK)                   tp0 (ACK)                   
			//       |<------------------------->|<------------------------->|
			//       |                           |                           |     
			//       |                           |                           |     
			//        ___________________________                  .......... ____________...
			//       |                           |                 .         |       
			//       |                           |                 .         |        
			//       |                           |                 .         |        
			//       |                           |                 .         |        
			//       |                           |                 .     85% |        
			//       |                           |                 .      :  |        
			//       |                           |            50%  .  80% :  |        
			//  ...__|                           |___________________________|        
			//                                   :             :       :  :  :        
			//                                   :             :       :  :  :        
			//                                   :             :       :  :  :        
			//                                   :             :       :  :  :        
			//                                   :<----------->:<----->:  :  :        
			//                                   :                        :        
			//      xx% of tp0                   :      :                 :        :
			//                                   :  25% :     85%-25%     :        :
			//                                   :<---->:<--------------->:<------>:   
			//                                         (1.)             (2.)   100%-85%+25% (= 40%)
			//                                                                     
			// (1.),(2.) -> check signal value here                                                                     
			// ----------------------------------------------------------------------------
			// OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE OBSOLOETE 
			
			
			read_test_signal_1 = (ack_lenght_in_cpu_cycles*(0.25)				)/11;	// 1 delay loop takes 11 cycles,  1265 cycles for computation of this line
			read_test_signal_2 = (ack_lenght_in_cpu_cycles*(0.85-0.25)			)/11;	// 1 delay loop takes 11 cycles,  1265 cycles for computation of this line
			read_test_signal_3 = (ack_lenght_in_cpu_cycles*(1.00-0.85+0.25)	)/11;	// 1 delay loop takes 11 cycles,  1265 cycles for computation of this line
			
			delay_loops_long( read_test_signal_1 );					// 1 delay loop takes 11 cycles
			
			
			for(i=0;i<14;i++)
			{	
				first_reading=0; second_reading=0;
				(*hal815_data_out)<<=1;
				
				if( (IN(PORT_HAL815_IN) & (1 << PIN_HAL815_IN)) )	// if bit is set
					first_reading=1;
				delay_loops_long( read_test_signal_2 );				// 1 delay loop takes 11 cycles
				
				if( (IN(PORT_HAL815_IN) & (1 << PIN_HAL815_IN)) )	// if bit is set
					second_reading=1;		
				delay_loops_long( read_test_signal_3 );				// 1 delay loop takes 11 cycles

				if(first_reading!=second_reading)
				{
					*hal815_data_out |= 1;
				}
				
			}
			// TODO also read Data Parity Bit (DP)

		}
		#else
			get_sig_lenght_in_cpu_cycles = 0;
			in_bit_counter=0;
			hal815_data_out_=0;
			PCICR |= (1<<PCIE1);	// Pin Change Interrupt Enable 1
			PCMSK1 |= (1<<PCINT8);	// PC ==> ADC0 (ADC Input Channel 0)  ==>  PCINT8 (Pin Change Interrupt 8)
			timer1Init(); 			// init+start timer
			timer1_measurement_running = 1; 
			
			
			
			while(timer1_measurement_running) // deactiveated by timer1 overflow interrupt
			{
				asm volatile("nop");
			}
			
			TCCR1B = 0;				// stop the timer
			TCNT1 = 0;				// reset TCNT1
			PCICR &= ~(1<<PCIE1);	// Pin Change Interrupt Disable 1
			errflag=0;
			
			*hal815_data_out = hal815_data_out_;
				
		#endif
		
	}
		
		
	if((hal815_com & 0x07) == COMMAND_WRITE)
	{	
		serialword |= (uint32_t)(          		hal815_data_in & 0x3FFF)		      	<<(32-3-1-4-1-14);		// 14bits	Data Bits (DAT)
		serialword |= (uint32_t)( (~(UNEVEN(	hal815_data_in & 0x3FFF,14))) 	& 0x01)	<<(32-3-1-4-1-14-1);	// 1bit		Data Parity Bit (DP)
		
		send_voltagemodulation(NSERIAL_WORD_BITS_WRITE, serialword);
			
		if(hal815_wait4ack(&ack_lenght_in_cpu_cycles))
			errflag = 1;	// write failed

	}
	
	
	if((hal815_com & 0x07) == COMMAND_ERASE || \
	   (hal815_com & 0x07) == COMMAND_PROM  || \
	   (hal815_com & 0x07) == COMMAND_LOCKI || \
	   (hal815_com & 0x07) == COMMAND_LOCK  )	
	{
		send_voltagemodulation(NSERIAL_WORD_BITS_ERASE, serialword);
	
		if(hal815_wait4ack(&ack_lenght_in_cpu_cycles))
		{
			errflag = 1;	// write failed
		}
		else
		{
			// Delay time of programming voltage after Acknowledge
			delay_loops_long( DELAY_LOOPS( HAL815_TW    )/11 );			// delay loop takes 11 cycles
		
			OUT(PORT_HAL815_OUT_EEPROM) |=  (1<<PIN_HAL815_OUT_EEPROM);	// set pin to 12V
			delay_loops_long( DELAY_LOOPS( HAL815_TPROG )/11 );			// delay loop takes 11 cycles		
			OUT(PORT_HAL815_OUT_EEPROM) &= ~(1<<PIN_HAL815_OUT_EEPROM);	// set pin to 5.5V
		}
	
	}
	
	
	rprintfCRLF(); rprintfProgStrM(" serialword      : ");  	rprintfNum( 2,32, FALSE,'0', serialword );

	get_ack_lenght_in_cpu_cycles = ack_lenght_in_cpu_cycles; // test value
	return errflag;				// return write status

}



//############################################################################
//! write data to device 
uint8_t hal815_wait4ack(uint32_t* ack_lenght_in_cpu_cycles)
//############################################################################
{

	uint8_t errflag = 0;
	uint16_t cntr = 0;
	*ack_lenght_in_cpu_cycles = 0;
	
	// TODo umschalten von ADC-eingang auf digitaleingang???
	
	
	while ( !(IN(PORT_HAL815_IN) & (1 << PIN_HAL815_IN))   && cntr<0XFFFF )  cntr++ ; 	// wait IN high  ( one while-loop takes 30 cpu cycles )
	if(cntr!=0XFFFF)
	{
		cntr = 0;
		while (  (IN(PORT_HAL815_IN) & (1 << PIN_HAL815_IN))   && cntr<0XFFFF )  cntr++ ; // wait IN low   ( one while-loop takes 30 cpu cycles )
	} 
	if(cntr==0XFFFF)
	{
		*ack_lenght_in_cpu_cycles = 0;
		errflag=1;								// write failed	
	}
	else
	{
		*ack_lenght_in_cpu_cycles = cntr*30;	// ( one while-loop took 30 cpu cycles, see above )
		errflag=0;								// write successful	
	}

	return errflag;								// return write status

}





//############################################################################
//! write data to device 
void send_voltagemodulation(uint8_t nbits, uint32_t data)
//############################################################################
{
	uint8_t i;

	// clear global interrupt flag
	cli();
	
	// send SYNC bit
	OUT(PORT_HAL815_OUT_WRITE) |=  (1<<PIN_HAL815_OUT_WRITE);	// set pin to 8V
	delay_loops_long( DELAY_LOOPS( HAL815_TP0 )/11 );			// delay loop takes 11 cycles
	OUT(PORT_HAL815_OUT_WRITE) &= ~(1<<PIN_HAL815_OUT_WRITE);	// set pin to 5.5V


	// send nbits of data
	for(i=nbits;i>0;i--)
	{	
		if(data&0x80000000)
		{
			delay_loops_long( DELAY_LOOPS( HAL815_TP1 )/11 );
			OUT(PORT_HAL815_OUT_WRITE) ^= (1<<PIN_HAL815_OUT_WRITE);	// toggle pin
			delay_loops_long( DELAY_LOOPS( HAL815_TP2 )/11 );
		}
		else 
		{
		  delay_loops_long( DELAY_LOOPS( HAL815_TP0 )/11 );
		}
		OUT(PORT_HAL815_OUT_WRITE) ^= (1<<PIN_HAL815_OUT_WRITE);		// toggle pin
		
		data<<=1;								
	}
	
	// reset output
	//OUT(PORT_HAL815_OUT_WRITE) &= ~(1<<PIN_HAL815_OUT_WRITE); 		// set 8V  pin to 5.5V
	//OUT(PORT_HAL815_OUT_EEPROM) &= ~(1<<PIN_HAL815_OUT_EEPROM);	// set 12V pin to 5.5V
	sei();
}





//############################################################################
//! 
ISR(PCINT1_vect)
//############################################################################
{
	// PCICR |= (1<<PCIE1);		// Pin Change Interrupt Enable 1
	// PCMSK1 |= (1<<PCINT8);	// PC ==> ADC0 (ADC Input Channel 0)  ==>  PCINT8 (Pin Change Interrupt 8)
	// timer1Init(); 			// init timer
	//
	// TCCR1B = 0;				// stop the timer
	// PCICR &= ~(1<<PCIE1);	// Pin Change Interrupt Disable 1
	
	get_sig_lenght_in_cpu_cycles = TCNT1;
	TCNT1 = 0;
	uint32_t T = get_sig_lenght_in_cpu_cycles/(F_CPU/1000000);  // [s]	

	
	
	if((T>800) &&  (in_bit_counter<100))	// > 800 s
	{
		if((T>=2250) && (in_bit_counter==0))
		{
			ack_lenght_in_ms = T;
			in_bit_counter++;
		}
		else
		{
			if(T<=1400)  // T < xxx[s]
			{
				//in_bit_counter--;
			}
			if(T>=2250)
			{	
				hal815_data_out_ &= ~1;
				hal815_data_out_<<=1;
				in_bit_counter++;
			}
			if((T<2250) && (T>1400)) 
			{
				hal815_data_out_ |= 1;
				hal815_data_out_<<=1;
				in_bit_counter++;
			}
		}
	}
	#ifdef DEBUG_HAL815
		rprintfCRLF(); rprintfNum( 10,6, FALSE,' ', T );	
	#endif


	if(in_bit_counter==14+1){
		hal815_data_out_>>=1;
	//	rprintfCRLF(); rprintfNum( 10,6, FALSE,' ',hal815_data_out_ );
		in_bit_counter=100;
	}
}


//############################################################################
//! stop measurement
ISR(TIMER1_OVF_vect)
//############################################################################
 {
 	timer1_measurement_running=0;
 }













