//############################################################################
//
// File Name	: 'hal815.h'
// Title		: HAL815 - programmierbarer linearer Magnetfeldsensor 
//                         
// Author		: Zoltan Gradwohl
// Created		: 03/10/2008
// Revised		: 03/10/2008
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
//
//
//
//############################################################################

#ifndef HAL815_H
#define HAL815_H


#define PIN_HAL815_IN     			0		//  Pin  (ADC-Pin)  
#define PORT_HAL815_IN        		PORTC 	//  Port (ADC-Port)
#define PIN_HAL815_OUT_WRITE     	1		//  Pout1 (8V)   
#define PORT_HAL815_OUT_WRITE       PORTB 	//  Port
#define PIN_HAL815_OUT_EEPROM     	2		//  Pout2 (12V)     
#define PORT_HAL815_OUT_EEPROM      PORTB 	//  Port


#define RANGE_PLUS_MINUS_30MT	0	// -30 mT...30 mT
#define RANGE_PLUS_MINUS_40MT	4	// -40 mT...40 mT
#define RANGE_PLUS_MINUS_60MT	5	// -60 mT...60 mT
#define RANGE_PLUS_MINUS_75MT	1	// -75 mT...78 mT
#define RANGE_PLUS_MINUS_80MT	6	// -80 mT...80 mT
#define RANGE_PLUS_MINUS_90MT	2	// -90 mT...90 mT
#define RANGE_PLUS_MINUS_100MT	7	// -100 mT...100 mT
#define RANGE_PLUS_MINUS_150MT	3	// -150 mT...150 mT

#define FILTER_80HZ				0	// 80Hz (-3 dB Frequency)
#define FILTER_160HZ			1	// 160Hz (-3 dB Frequency)
#define FILTER_500HZ			2	// 500Hz (-3 dB Frequency)
#define FILTER_1000HZ			3	// 1000Hz (-3 dB Frequency)
#define FILTER_2000HZ			4	// 2000Hz (-3 dB Frequency)


#define TC_PLUS_400				31	// TODO ergnzen
#define TC_PLUS_300				28
#define TC_MINUS_810			4


#define COMMAND_READ			2	// read a register
#define COMMAND_WRITE			3	// write a register
#define COMMAND_PROM			4	// program all nonvolatile registers (except the lock bits)
#define COMMAND_ERASE			5	// erase all nonvolatile registers (except the lock bits)
#define COMMAND_LOCKI			6	// lock Micronas lockable register	
#define COMMAND_LOCK			7	// lock the whole device and switch permanently to the analog-mode

									//Data Bits		Format				Customer			Rema
#define REGADDR_CLAMP_LOW		1	// 10			binary				read/write/program	Low clamping voltage
#define REGADDR_CLAMP_HIGH		2	// 11			binary				read/write/program	High clamping voltage
#define REGADDR_VOQ				3	// 11			two compl.binary	read/write/program
#define REGADDR_SENSITIVITY		4	// 14			signed binary		read/write/program
#define REGADDR_MODE			5	// 6			binary				read/write/program	Range and filter settings
#define REGADDR_LOCKR			6	// 1			binary				lock 				Lock Bit
#define REGADDR_ADC_READOUT		7	// 14			two compl. binary	read
#define REGADDR_TC				11	// 6			signed binary		read/write/program
#define REGADDR_TCSQ			12	// 5			binary				read/write/program
#define REGADDR_DEACTIVATE		15	// 12			binary				write				Deactivate the sensor



#define HAL815_TP1_50PERCENT	0.50	// [%/100]	 min % of tp0 or tpOUT
#define HAL815_TP1_65PERCENT	0.65	// [%/100]	 typ % of tp0 or tpOUT
#define HAL815_TP1_80PERCENT	0.80	// [%/100]	 max % of tp0 or tpOUT
#define HAL815_TP0				1.75	// [ms]	 Bit time on VDD
#define HAL815_TPOUT			3.00	// [ms]  Bit time on output pin
#define HAL815_TP1				(HAL815_TP1_65PERCENT * HAL815_TP0)	// [ms]  Voltage Change for logical 1 (65% of Bit time on VDD)
#define HAL815_TP2				(HAL815_TP0-HAL815_TP1)	// [ms]  
#define HAL815_TPROG			100		// [ms]  Programming Time for EEPROM
#define HAL815_TACT				0.1		// [ms]  Duration of an Activate pulse
#define HAL815_TW				0.7		// [ms]  Delay time of programming voltage after Acknowledge



#define NSERIAL_WORD_BITS_WRITE (3+1+4+1+14+1)
#define NSERIAL_WORD_BITS_READ  (3+1+4+1) 
#define NSERIAL_WORD_BITS_ERASE (3+1+4+1) 



  //  -- really weird C code to count the number of bits in a word
#define BITCOUNT(x)     (((BX_(x)+(BX_(x)>>4)) & 0x0F0F0F0F) % 255)
#define BX_(x)          ((x) - (((x)>>1)&0x77777777)   \
                        - (((x)>>2)&0x33333333)        \
                        - (((x)>>3)&0x11111111))
#define	UNEVEN(x,n)		(((n)-BITCOUNT(x)) & 1)

#define DELAY_LOOPS(x)	((uint32_t)( (F_CPU)*((x)/1000.0) ))



// Prototypes
void hal815_init(void);
uint8_t hal815_wait4ack(uint32_t* ack_lenght_in_cpu_cycles);
uint8_t hal815_com(uint8_t hal815_com, uint8_t hal815_adr, uint16_t hal815_data_in, uint16_t* hal815_data_out);
void send_voltagemodulation(uint8_t nbits, uint32_t data);

#endif
