
//############################################################################
//
// File Name	: 'main.c'
// Title		: HAL815 - programmierbarer linearer Magnetfeldsensor 
//                         
// Author		: Zoltan Gradwohl
// Created		: 03/10/2008
// Revised		: 03/10/2008
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
//
//
//
//#############################################################################

// Include Files 
#include <avr/io.h>			// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <string.h>			// include string support

#include "global.h"			// include our global settings
#include "uart.h"			// include uart function library
#include "timer.h"			// include timer function library (timing, PWM, etc)
#include "main.h"			// include header
#include "vt100.h"			// include VT100 terminal commands
#include "rprintf.h"		// include printf function library
#include "hal815.h"			// include printf function library
#include "a2d.h"			// include analog to digital functions



int c = 0;
extern uint32_t get_ack_lenght_in_cpu_cycles;
extern uint16_t ack_lenght_in_ms;


//############################################################################
// Begin Code 
int main(void)
//############################################################################
{
	
	/*hal815_init();
	//uint16_t ack_lenght_in_loops;
	//hal815_wait4ack(&ack_lenght_in_loops);
	volatile uint8_t errflag = 0;
	uint16_t hal815_data_out;
	//hal815_com(COMMAND_WRITE, REGADDR_MODE, RANGE_PLUS_MINUS_150MT, &hal815_data_out);
	errflag = hal815_com(COMMAND_READ, REGADDR_MODE, RANGE_PLUS_MINUS_150MT, &hal815_data_out);
	*/
	/*
	volatile uint8_t temp=0;
	temp|= 1;
	temp&=~1;
	temp|= 1;
	temp<<=1;
	temp<<=1;
	*/
	/*
	volatile uint16_t	temp1=0xE000; 
	volatile uint16_t	temp = (temp1&0xE000)>>13;
						temp = (temp1&0x1C00)>>10;				
	*/
	/*
	volatile uint32_t	temp1= DELAY_LOOPS( HAL815_TPROG )/11;
	delay_loops_long( temp1 );			// delay loop takes 11 cycles
	*/
	
	/*
	
	volatile uint8_t i=0;
	volatile uint8_t nbits=4;
	volatile uint8_t temp=0;
	for(i=nbits;i>0;i--)
	{
		if(i>1)
			temp++;
	}
	
	*/
	
	
	
	
	
	
	
	
	
	
	
	//uint8_t err_flag=0;

	// initialize the UART (serial port)
	uartInit();
 
	// make all rprintf err_flagments use uart for output
	rprintfInit(uartSendByte);

	// initialize vt100 terminal
	vt100Init();
	
	// wait x-s
	//delay_loops_long(10000000);

	// print welcome message
	print_welcome_msg();			
		
 
	// initialize AD8555 port (1-wire interface)
	hal815_init();
	rprintfProgStrM(    " HAL815 ports initialized............................... OK \r\n");
	
	// initialize ADC 
	a2dInit();
	a2dStartConvert_freerun(ADC_CH_ADC0);		// start ADC - conversion
	ADCSRA &= ~(1<<ADIE);						// disable ADC interrupts -> enabled by int0
	rprintfProgStrM(    " ADC channel-0 initialized ............................. OK \r\n");
	

	rprintfProgStrM("-------------------------------------------------------------\r\n");
	// print message
	rprintfProgStrM("#############################################################\r\n");
	

	// The rising edge of INT0 generates an interrupt request.
	// MCUCR = (1<<ISC01)|(1<<ISC00);
	// INT0: External Interrupt Request 0 Enable
	// GICR = (1<<INT0);

	// enable global interrupt
	sei();

	// print message
	//rprintfProgStrM("#############################################################\r\n");
	rprintfCRLF(); rprintfProgStrM("--Starting Programm--\r\n");  
	rprintfProgStrM(">> "); 
	// empty receive buffer
	uartFlushRxBuffer(); 
	
	// run function
	run_programm();
	
	return 0;
}



//############################################################################
void run_programm(void)
//############################################################################
{
	int8_t c0=0, c1=0; 
	uint8_t errflag=0, user_abort=0;
	uint16_t hal815_data_out=0, hal815_data_in=0;
	uint16_t mode_nr=0, filter_nr=0;
	uint16_t register_nr=0, command_nr=0;
	uint16_t clamp_low=0, clamp_high=0;
	int16_t  v0q=0;
	int16_t  sensitivity=0;
	int16_t  adc_readout=0;
	uint16_t quadratic_temperature_coefficient=0;
	int16_t	 temperature_coefficient=0;
										
										

	while(1)
	{
		
			// TEST: TODO delete
		//	OUT(PORT_HAL815_OUT_WRITE) ^= (1<<PIN_HAL815_OUT_WRITE);	// toggle pin
		//	delay_loops_long( DELAY_LOOPS( HAL815_TP1 )/11 );
		//	OUT(PORT_HAL815_OUT_EEPROM) ^=  (1<<PIN_HAL815_OUT_EEPROM);	// toggle pin
		//	delay_loops_long( DELAY_LOOPS( HAL815_TP1 )/11 );
	
	
		// check for keypress
		if((c=uartGetByte()) != -1)
		{
			// echo command
			rprintfChar(c);	rprintfCRLF(); 
			c0 = 0; c1= 0;

			switch(c)
			{
				//---------------------------------------------
				// print welcome message
				//---------------------------------------------
				case 'p':
				{
					print_welcome_msg();
					//rprintfProgStrM("#############################################################\r\n");
					break;
				}

				//---------------------------------------------
				// setup hal815
				//---------------------------------------------
				case 's':
				{
					print_welcome_msg();
					
					// reset values
					user_abort=0;
					hal815_data_in=0; hal815_data_out=0;	
					mode_nr=0; filter_nr=0;				
					clamp_low=0; clamp_high=0; v0q=0;
					register_nr=0, command_nr=0;
					v0q=0;
					sensitivity=0;
					adc_readout=0;
					quadratic_temperature_coefficient=0;
					temperature_coefficient=0;
					
					
					
					rprintfProgStrM("Please select a command (You can abort pressing ESC):\r\n");
					rprintfProgStrM("  1.) read a register\r\n");
					rprintfProgStrM("  2.) write a register\r\n");
					rprintfProgStrM("  3.) program all nonvolatile registers (except the lock bits)\r\n");
					rprintfProgStrM("  4.) erase all nonvolatile registers (except the lock bits)\r\n");
					rprintfProgStrM("  5.) lock Micronas lockable register\r\n");
					rprintfProgStrM("  6.) lock the whole device and switch permanently to the analog-mode\r\n");
					rprintfProgStrM(">> ");  

					if(!get_num(6, &command_nr)) 
					{
						command_nr = (command_nr < 1) ? 1 : command_nr; // minimum ist 1
						//rprintfCRLF(); rprintfProgStrM("Number entered: "); rprintfNum( 10,6, FALSE,' ', command_nr );  
						rprintfCRLF();rprintfCRLF();
					 
						switch(command_nr)
						{
							case(1):
								command_nr = COMMAND_READ;
							break;
							case(2):
								command_nr = COMMAND_WRITE;
							break;	
							case(3):
								command_nr = COMMAND_PROM;
							break;
							case(4):
								command_nr = COMMAND_ERASE;
							break;	
							case(5):
								command_nr = COMMAND_LOCKI;
							break;	
							case(6):
								rprintfProgStrM(" !!! Do you really want to lock all registers (the sensor will\r\n"); 
								rprintfProgStrM("     no longer respond to any supply voltage modulation) [y/n]\r\n");
							
								while(c!='y' && c!='n'){
									while((c=uartGetByte()) == -1){};
									if(c=='y'){
										command_nr = COMMAND_LOCK;
									}
									if(c=='n'){
										rprintfChar(c); rprintfCRLF();
										rprintfProgStrM(" Sensor NOT locked.\r\n>>");
										user_abort=1;	
									}
								};
							break;	
						}


						if((command_nr == COMMAND_READ) || (command_nr == COMMAND_WRITE))
						{
							uint8_t nr_of_entries=7;
							rprintfProgStrM("Please select an address:\r\n");
							rprintfProgStrM("  1.) Low clamping voltage\r\n");
							rprintfProgStrM("  2.) High clamping voltage\r\n");
							rprintfProgStrM("  3.) Output Quiescent Voltage (V0Q) \r\n");
							rprintfProgStrM("  4.) Sensitivity\r\n");
							rprintfProgStrM("  5.) Mode\r\n");
							rprintfProgStrM("  6.) Temperature (TC)\r\n");
							rprintfProgStrM("  7.) Temperature (TCSQ)\r\n"); 
							if(command_nr==COMMAND_READ)
							{
								rprintfProgStrM("  8.) ADC-readout\r\n");
								nr_of_entries = 8;
							}
							if(command_nr==COMMAND_WRITE)
							{
								rprintfProgStrM("  8.) Deactivate the sensor\r\n");
								nr_of_entries = 8;
							}
							rprintfProgStrM(">> ");
						
						
							if(!get_num(nr_of_entries, &register_nr)) 
							{
								register_nr = (register_nr < 1) ? 1 : register_nr; // minimum ist 1
								//rprintfCRLF(); rprintfProgStrM("Number entered: "); rprintfNum( 10,6, FALSE,' ', register_nr );  rprintfCRLF();
								rprintfCRLF();rprintfCRLF();
								
								switch(register_nr)
								{
									case(1):
										register_nr = REGADDR_CLAMP_LOW;
										if(command_nr == COMMAND_WRITE)
										{      
											rprintfProgStrM("Please enter clamp-low[0...1023]:\r\n");
											rprintfProgStrM(">> "); 
											if(!get_num(1023, &clamp_low))
											{
												rprintfCRLF(); rprintfProgStrM("Value entered: "); rprintfNum( 10,6, FALSE,' ', clamp_low );  rprintfCRLF();
												hal815_data_in = clamp_low;
												
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}	
									break;
									case(2):
										register_nr = REGADDR_CLAMP_HIGH;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please enter clamp-high[0...2047]:\r\n");
											rprintfProgStrM(">> ");  
											if(!get_num(2047, &clamp_high))
											{
												rprintfCRLF(); rprintfProgStrM("Value entered: "); rprintfNum( 10,6, FALSE,' ', clamp_high );  rprintfCRLF();
												hal815_data_in = clamp_high;
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;	
									case(3):
										register_nr = REGADDR_VOQ;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please enter VOQ[-1024...1023]:\r\n");		// TODO negative werte
											rprintfProgStrM(">> "); 
											if(!get_int_num(-1024, 1023, &v0q))
											{
												rprintfCRLF(); rprintfProgStrM("Value entered: "); rprintfNum( 10,6, TRUE,' ', v0q );  rprintfCRLF();
												
												// 2er komplement fr eine 11 bit zahl
												if(v0q&0x0400){   // val &  0000 0100 0000 0000  (11 bit int wert) vorzeichen prfen
												   v0q&=0x07FF;   // val &= 0000 0111 1111 1111  (11 bit int wert)
												}
												
												hal815_data_in = v0q;
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;
									case(4):
										register_nr = REGADDR_SENSITIVITY;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please enter sensitivity[-8192...8191]:\r\n");		// TODO negative werte
											rprintfProgStrM(">> "); 
											if(!get_int_num(-8192, 8191, &sensitivity))
											{
												#define DATA_BITS_SENSITIVITY 14
												if(sensitivity<0)
													sensitivity = (ABS(sensitivity) | (1<<(DATA_BITS_SENSITIVITY-1)));
													
												hal815_data_in = sensitivity;
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;	
									case(5):
										register_nr = REGADDR_MODE;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please select the magnetic field range:\r\n");
											rprintfProgStrM("  1.) +-30 mT\r\n");
											rprintfProgStrM("  2.) +-40 mT\r\n");
											rprintfProgStrM("  3.) +-60 mT\r\n");
											rprintfProgStrM("  4.) +-75 mT\r\n");
											rprintfProgStrM("  5.) +-80 mT\r\n");
											rprintfProgStrM("  6.) +-90 mT\r\n");
											rprintfProgStrM("  7.) +-100 mT\r\n");
											rprintfProgStrM("  8.) +-150 mT\r\n");
											rprintfProgStrM(">> "); 
											if(!get_num(8, &mode_nr))
											{
												rprintfCRLF(); rprintfProgStrM("Value entered: "); rprintfNum( 10,6, FALSE,' ', mode_nr );  rprintfCRLF();
												switch(mode_nr)
												{
													case(1):
														mode_nr = RANGE_PLUS_MINUS_30MT;
													break;
													case(2):
														mode_nr = RANGE_PLUS_MINUS_40MT;
													break;	
													case(3):
														mode_nr = RANGE_PLUS_MINUS_60MT;
													break;
													case(4):
														mode_nr = RANGE_PLUS_MINUS_75MT;
													break;	
													case(5):
														mode_nr = RANGE_PLUS_MINUS_80MT;
													break;	
													case(6):
														mode_nr = RANGE_PLUS_MINUS_90MT;
													break;	
													case(7):
														mode_nr = RANGE_PLUS_MINUS_100MT;
													break;	
													case(8):
														mode_nr = RANGE_PLUS_MINUS_150MT;
													break;	
												}
												
												
												rprintfProgStrM("Please select the filter:\r\n");
												rprintfProgStrM("  1.) 80  Hz\r\n");
												rprintfProgStrM("  2.) 160 Hz\r\n");
												rprintfProgStrM("  3.) 500 Hz\r\n");
												rprintfProgStrM("  4.) 1000 Hz\r\n");
												rprintfProgStrM("  5.) 2000 Hz\r\n");
												rprintfProgStrM(">> "); 
												if(!get_num(5, &filter_nr)) 
												{
													rprintfCRLF(); rprintfProgStrM("Value entered: "); rprintfNum( 10,6, FALSE,' ', filter_nr );  rprintfCRLF();
													switch(filter_nr)
													{
														case(1):
															filter_nr = FILTER_80HZ;
														break;
														case(2):
															filter_nr = FILTER_160HZ;
														break;	
														case(3):
															filter_nr = FILTER_500HZ;
														break;
														case(4):
															filter_nr = FILTER_1000HZ;
														break;	
														case(5):
															filter_nr = FILTER_2000HZ;
														break;	
													}
													
													// In the Write command, the last bits are valid. If, for
													// example, the TC register (6 bits) is written, only the
													// last 6 bits are valid.
													//
													// connect high-bits and low-bits
													hal815_data_in = ( ((filter_nr&0x07)<<3)   |   (mode_nr&0x07) );

												}
												else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
												
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;	
									case(6):
										register_nr = REGADDR_TC;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please select TC[-31...31]:\r\n");		// TODO negative werte
											rprintfProgStrM(">> "); 
											if(!get_int_num(-31, 31, &temperature_coefficient))
											{
												#define DATA_BITS_TC 6
												if(temperature_coefficient<0)
													temperature_coefficient = (ABS(temperature_coefficient) | (1<<(DATA_BITS_TC-1)));
													
												hal815_data_in = temperature_coefficient;
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;	
									case(7):
										register_nr = REGADDR_TCSQ;
										if(command_nr == COMMAND_WRITE)
										{ 
											rprintfProgStrM("Please select TCSQ[0...31]:\r\n");		// TODO negative werte
											rprintfProgStrM(">> "); 
											if(!get_num(31, &quadratic_temperature_coefficient))
											{
												hal815_data_in = quadratic_temperature_coefficient;
											}
											else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
										}
									break;	
									case(8):
										if(command_nr==COMMAND_READ)
										{
											register_nr = REGADDR_ADC_READOUT;
										}
										if(command_nr==COMMAND_WRITE)
										{
											register_nr = REGADDR_DEACTIVATE;
											
											hal815_data_in = 2063;
										}
									break;		
								}
							}else{rprintfProgStrM(" User abort.\r\n"); user_abort=1;}
							
						}	
						else // ERASE, PROM, LOCK, and LOCKI
						{
							// If the content of any register (except the lock registers)
							// is to be changed, the desired value must first be written
							// into the corresponding RAM register. Before reading
							// out the RAM register again, the register value must
							// be permanently stored in the EEPROM.
							
							// Permanently storing a value in the EEPROM is done
							// by first sending an ERASE command followed by
							// sending a PROM command. The address within the
							// ERASE and PROM commands is not important.
							// ERASE and PROM act on all registers in parallel.
							
							// If all HAL 815 registers are to be changed, all writing
							// commands can be sent one after the other, followed by
							// sending one ERASE and PROM command at the end.

							register_nr = 0;
							hal815_data_in = 0;
							//user_abort=0;

						}
						
						
						
						
	
						if(!user_abort)
						{
							
											   rprintfProgStrM("-------- Communication data -----------"); 
							rprintfCRLF();     rprintfProgStrM(" command_nr      :");  	rprintfNum( 10,6, FALSE,' ', command_nr );  	
							rprintfCRLF();     rprintfProgStrM(" register_nr     :");  	rprintfNum( 10,6, FALSE,' ', register_nr );  	
							rprintfCRLF();     rprintfProgStrM(" hal815_data_in  :");  	rprintfNum( 10,6, FALSE,' ', hal815_data_in ); 
							
							// send data to device
							errflag = hal815_com(command_nr, register_nr, hal815_data_in, &hal815_data_out);
							if(command_nr == COMMAND_WRITE){
								rprintfCRLF(); rprintfProgStrM(" write ack-length:");  	rprintfNum( 10,6, FALSE,' ', get_ack_lenght_in_cpu_cycles); rprintfProgStrM(" cpu-cycles"); }
							if(command_nr == COMMAND_READ){
								rprintfCRLF(); rprintfProgStrM(" read ack-length :");  	rprintfNum( 10,6, FALSE,' ', ack_lenght_in_ms); rprintfProgStrM(" us");}
							rprintfCRLF();     rprintfProgStrM(" return value is :");		rprintfNum( 10,6, FALSE,' ', hal815_data_out );  
							rprintfCRLF();     rprintfProgStrM("---------------------------------------");rprintfCRLF(); 
							

							
							if(!errflag)
							{
								if(command_nr == COMMAND_READ)
								{	
									
									if(register_nr == REGADDR_CLAMP_LOW)
									{      
										clamp_low = (hal815_data_out>>4); // signal length 10 bits: shift 4 to rigth
										rprintfCRLF(); rprintfProgStrM("clamp-low is: ");  rprintfNum( 10,6, TRUE,' ', clamp_low );  rprintfCRLF();
									}
										
									if(register_nr == REGADDR_CLAMP_HIGH)
									{      
										clamp_high = (hal815_data_out>>3); // signal length 11 bits: shift 3 to rigth
										rprintfCRLF(); rprintfProgStrM("clamp-high is: "); rprintfNum( 10,6, TRUE,' ', clamp_high );  rprintfCRLF();
									}	
									
									
									// In the Read command, the first bits are valid. If, for
									// example, the TC register (6 bits) is read, only the first 6
									// bits are valid.
									 
									if(register_nr == REGADDR_VOQ)
									{	
										v0q =(hal815_data_out>>3); // signal length 11 bits: shift 3 to rigth 
										
										// 2er komplement fr eine 11 bit zahl
										if(v0q&0x0400){   // val &  0000 0100 0000 0000  (11 bit int wert) vorzeichen prfen
											v0q|=0xF800;  // val &= 1111 1000 0000 0000  (11 bit int wert)
										}
										rprintfCRLF(); rprintfProgStrM("VOQ is: "); rprintfNum( 10,6, TRUE,' ', v0q );  rprintfCRLF();
									}
									
									
									if(register_nr == REGADDR_SENSITIVITY)
									{
										sensitivity = (hal815_data_out>>0); // signal length 14 bits: shift 0 to rigth 
										
										// Signed binary number einer 14 bit zahl
										if(sensitivity & (1<<(DATA_BITS_SENSITIVITY-1)))
											sensitivity = -(sensitivity&0x1FFF); 	// 0x1FFF == 111 1111 1111 11 (->13x1)
										rprintfCRLF(); rprintfProgStrM("Sensitivity is: "); rprintfNum( 10,6, TRUE,' ', sensitivity );  rprintfCRLF();
									}
									
	
									if(register_nr == REGADDR_MODE) 
									{
										// disconnect high-bits and low-bits
										filter_nr = (hal815_data_out&0x3800)>>11;
										mode_nr   = (hal815_data_out&0x0700)>>8;								
										rprintfCRLF(); //rprintfProgStrM("filter_nr:"); rprintfNum( 10,6, FALSE,' ', filter_nr );  	
										//rprintfCRLF(); rprintfProgStrM("mode_nr:");   rprintfNum( 10,6, FALSE,' ', mode_nr );rprintfCRLF(); 
										
										rprintfProgStrM("The filter setting is ");	
										switch(filter_nr)
										{
											case(FILTER_80HZ):
												rprintfProgStrM("80 Hz\r\n");
											break;
											case(FILTER_160HZ):
												rprintfProgStrM("160 Hz\r\n");
											break;	
											case(FILTER_500HZ):
												rprintfProgStrM("500 Hz\r\n");
											break;
											case(FILTER_1000HZ):
												rprintfProgStrM("1000 Hz\r\n");
											break;	
											case(FILTER_2000HZ):
												rprintfProgStrM("2000 Hz\r\n");
											break;	
										}
										rprintfProgStrM("The magnetic field range is ");
										switch(mode_nr)
										{
											case(RANGE_PLUS_MINUS_30MT):
												rprintfProgStrM("+-30 mT\r\n");
											break;
											case(RANGE_PLUS_MINUS_40MT):
												rprintfProgStrM("+-40 mT\r\n");
											break;	
											case(RANGE_PLUS_MINUS_60MT):
												rprintfProgStrM("+-60 mT\r\n");
											break;
											case(RANGE_PLUS_MINUS_75MT):
												rprintfProgStrM("+-75 mT\r\n");
											break;	
											case(RANGE_PLUS_MINUS_80MT):
												rprintfProgStrM("+-80 mT\r\n");
											break;	
											case(RANGE_PLUS_MINUS_90MT):
												rprintfProgStrM("+-90 mT\r\n");
											break;	
											case(RANGE_PLUS_MINUS_100MT):
												rprintfProgStrM("+-100 mT\r\n");
											break;	
											case(RANGE_PLUS_MINUS_150MT):
												rprintfProgStrM("+-150 mT\r\n");
											break;	
										}
									}
									
									
									if(register_nr == REGADDR_ADC_READOUT)
									{
										adc_readout = (hal815_data_out>>0); // signal length 14 bits: shift 0 to rigth 
										
										// 2er komplement fr eine 14 bit zahl
										if(adc_readout&0x2000){   // val &  0010 0000 0000 0000  (14 bit int wert) vorzeichen prfen
											adc_readout|=0xC000;  // val &= 1100 0000 0000 0000  (14 bit int wert)
										}
										rprintfCRLF(); rprintfProgStrM("ADC-Readout is (-8192 up to 8191): "); rprintfNum( 10,6, TRUE,' ', adc_readout );  rprintfCRLF();
									}
									
									
									if(register_nr == REGADDR_TC)
									{
										temperature_coefficient = (hal815_data_out>>8); // signal length 6 bits: shift 8 to rigth 
										
										// Signed binary number einer 14 bit zahl
										if(temperature_coefficient & (1<<(DATA_BITS_TC-1)))
											temperature_coefficient = -(temperature_coefficient&0x001F); 	// 0x001F == 111 11 (->5x1)
										rprintfCRLF(); rprintfProgStrM("Temperature Coefficient is: "); rprintfNum( 10,6, TRUE,' ', temperature_coefficient );  rprintfCRLF();
									}	
									
									
									if(register_nr == REGADDR_TCSQ)
									{
										quadratic_temperature_coefficient = (hal815_data_out>>9); // signal length 5 bits: shift 9 to rigth 
										rprintfCRLF(); rprintfProgStrM("Quadratic Temperature Coefficient is: "); rprintfNum( 10,6, FALSE,' ', quadratic_temperature_coefficient );  rprintfCRLF();
									}	
									
									
									rprintfCRLF();     rprintfProgStrM("---------------------------------------");rprintfCRLF(); 	
								}	
							}
							else
							{
								rprintfCRLF(); rprintfProgStrM(" ERROR Device not responding"); 
							}
							rprintfCRLF();
						}

						
						//#define REGADDR_CLAMP_LOW			1	// 10			binary				read/write/program	Low clamping voltage
						//#define REGADDR_CLAMP_HIGH		2	// 11			binary				read/write/program	High clamping voltage
						//#define REGADDR_VOQ				3	// 11			two compl.binary	read/write/program
						//#define REGADDR_SENSITIVITY		4	// 14			signed binary		read/write/program
						//#define REGADDR_MODE				5	// 6			binary				read/write/program	Range and filter settings
						//#define REGADDR_LOCKR				6	// 1			binary				lock 				Lock Bit
						//#define REGADDR_ADC_READOUT		7	// 14			two compl. binary	read
						//#define REGADDR_TC				11	// 6			signed binary		read/write/program
						//#define REGADDR_TCSQ				12	// 5			binary				read/write/program
						//#define REGADDR_DEACTIVATE		15	// 12			binary				write				Deactivate the sensor
					
						
						
						
						
						
					}
					else rprintfProgStrM(" User abort.\r\n");

					break;
				}
	
	
				//---------------------------------------------
				// hal815 - make a single measurement 
				//---------------------------------------------
				case 'm':
				{
					// setup local variable
					uint16_t adc_temp=0;

					// adc initialisieren
					ADCSRA=(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1);	//enable ADC single conversion	- /64
					// ADMUX = ADC_CH_ADC0; 					// select channel 	
					ADMUX = PIN_HAL815_IN; 						// select channel 	
			
			
					// startet adc messung
					ADCSRA|=(1<<ADSC); 	
					while(ADCSRA & (1<<ADSC)){;}				// 1. convcersion is trash
					adc_temp = ADC;	
					while(ADCSRA & (1<<ADSC)){;}				// 2. convcersion is ok
					adc_temp = ADC;	

				
					adc_temp =0;	
					// durchschnitt aus 16 messungen
					for(uint8_t i=0;i<16;i++)
					{
						ADCSRA |= (1<<ADSC);           		// eine Wandlung "single conversion"
						while(ADCSRA & (1<<ADSC)){;}
		
						adc_temp += ADC;		    			// Wandlungsergebnisse aufaddieren
					}
					//result /= 16;
					adc_temp >>=4;
	
					ADCSRA &= ~(1<<ADEN);             	// ADC deaktivieren	
		

					// ausgabe der ergebnisse	
					rprintfProgStrM(" ADC-Measured value:    "); rprintfNum( 10,6, FALSE,' ', adc_temp );                        rprintfProgStrM(" [-]\r\n");
				
					break;
				}
	
	
				//---------------------------------------------	
				// default
				//---------------------------------------------	
				default:
					break;
					
			}
	
			rprintfProgStrM(">> "); 
		}
			
		
		
	}

	
}





// rprintf 
//############################################################################
// wait for user input and create number "< max_range"
uint8_t get_num(uint16_t max_range, uint16_t *num)
//############################################################################
{
	int8_t chars[6]={0,0,0,0,0,0};  // 2^16 = 65536 -> max 5 numbers 
	uint8_t m=0,n=0;
	uint16_t faktor10[5] = {1,10,100,1000,10000};		
	
	do{
		// wait for keypress
		while((chars[m]=uartGetByte()) == -1){};	
		// check for valid numbers
		if((chars[m]>='0') && (chars[m]<='9') && (m<5))
			rprintfChar((uint8_t)chars[m++]);
			
	}while(chars[m]!=ASCII_ESC && chars[m]!=ASCII_CR);
	
	
	if(chars[m]!=ASCII_ESC){
		// calculate number out of char
		m--;
		*num = 0;
		for(n=0;n<=m;n++){
			*num += (uint16_t)((chars[n]-'0')*faktor10[m-n]);
		}
		if(*num >= max_range)
			*num = max_range;
		return(0);
	}

	return(1);
}


// rprintf 
//############################################################################
// wait for user input and create number "< max_range"
uint8_t get_int_num(int16_t min_range, int16_t max_range, int16_t *num)
//############################################################################
{
	int8_t chars[6]={0,0,0,0,0,0};  // 2^16 = 65536 -> max 5 numbers 
	uint8_t m=0,n=0;
	uint16_t faktor10[5] = {1,10,100,1000,10000};	
	int8_t num_sign = 1;
	
	do{
		// wait for keypress
		while((chars[m]=uartGetByte()) == -1){};	
		if((m==0) && (chars[0]=='-') && (num_sign==1)){
			num_sign = -1; m=0;
			rprintfChar('-');
		}	
		
		// check for valid numbers
		if((chars[m]>='0') && (chars[m]<='9') && (m<5))
			rprintfChar((uint8_t)chars[m++]);
			
	}while(chars[m]!=ASCII_ESC && chars[m]!=ASCII_CR);
	
	
	if(chars[m]!=ASCII_ESC){
		// calculate number out of char
		m--;
		*num = 0;
		for(n=0;n<=m;n++){
			*num += (uint16_t)((chars[n]-'0')*faktor10[m-n]);
		}
		(*num)*=num_sign;
		
		if(*num >= max_range)
			*num = max_range;
		if(*num <= min_range)
			*num = min_range;	
		return(0);
	}

	return(1);
}






	
	
	
// rprintf 
//############################################################################
// wait for user input and create string "< max_range"   
uint8_t get_str(uint16_t max_range, int8_t chars[30], uint8_t *m)
//############################################################################
{
	*m=0;
	do{
		// wait for keypress
		while((chars[*m]=uartGetByte()) == -1){};	
		// check for valid numbers
		if((chars[*m]>='A') && (chars[*m]<='z'))
			rprintfChar(chars[(*m)++]);
			
			
	}while(chars[*m]!=ASCII_ESC && chars[*m]!=ASCII_CR);
	
	
	if(chars[*m]!=ASCII_ESC){
	
		chars[*m] = 0x00;
		
		return(0);
	}

	return(1);
}




// rprintf 
//############################################################################
// print welcome message
void print_welcome_msg(void)
//############################################################################
{
	// print welcome message
	vt100ClearScreen();
	vt100SetCursorPos(1,0);
	rprintfCRLF();
	rprintfProgStrM("##############################################################\r\n");
	rprintfProgStrM("     Welcome to the HAL815 Programming Suite! \r\n");
	rprintfProgStrM("     Zoltan Gradwohl / WINAVR / Atmega168@11.059MHz   V1.00 \r\n");
	rprintfProgStrM("     Last compiled:  "); 	rprintfProgStrM(__DATE__);rprintfCRLF();
	rprintfProgStrM("##############################################################\r\n");
	rprintfProgStrM("     You can select between following options: \r\n");
	rprintfProgStrM("     p : print welcome message\r\n");
	rprintfProgStrM("     s : setup hal815\r\n");
	rprintfProgStrM("     m : measure adc value \r\n");
	rprintfProgStrM("##############################################################\r\n");
	rprintfProgStrM(" If the content of any register (except the lock registers) is\r\n");
	rprintfProgStrM(" to be changed, the desired value must first be written into  \r\n"); 
	rprintfProgStrM(" the corresponding RAM register. Before reading out the RAM   \r\n");  
	rprintfProgStrM(" register again, the register value must be permanently stored\r\n");
	rprintfProgStrM(" in the EEPROM.\r\n\n");
	
	rprintfProgStrM(" Permanently storing a value in the EEPROM is done by first   \r\n");
	rprintfProgStrM(" sending an ERASE command followed by sending a PROM command. \r\n");
	rprintfProgStrM(" The address within the ERASE and PROM commands is not        \r\n");
	rprintfProgStrM(" important. ERASE and PROM act on all registers in parallel.  \r\n\n");
							
	rprintfProgStrM(" If all HAL 815 registers are to be changed, all writing      \r\n");
	rprintfProgStrM(" commands can be sent one after the other, followed by sending\r\n");
	rprintfProgStrM(" one ERASE and PROM command at the end.                       \r\n");
	rprintfProgStrM("##############################################################\r\n");
	
	
}














